@@ -20,16 +20,28 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: pulse.c,v 1.3 2007-11-18 17:24:13 gcasse Exp $
+ * $Id: pulse.c,v 1.8 2008-02-08 10:01:09 hanke Exp $
  */
 
-
 /* debug */
 
 /* #define DEBUG_PULSE */
+
 #include <stdio.h>
 #include <stdarg.h>
 
+enum {
+  /* 100ms. 
+     If a greater value is set (several seconds), 
+     please update _pulse_timeout_start accordingly */
+  PULSE_TIMEOUT_IN_USEC = 100000,  
+
+  /* return value */
+  PULSE_OK = 0,
+  PULSE_ERROR = -1,
+  PULSE_NO_CONNECTION = -2
+};
+
 #ifdef DEBUG_PULSE
 
 #define ENTER(text) debug_enter(text)
@@ -74,7 +86,6 @@
 static FILE* fd_log = NULL;
 static const char* FILENAME = "/tmp/pulse.log";
 
-
 static void 
 debug_init()
 {
@@ -364,7 +375,7 @@ _pulse_free(AudioID *id, size_t* length) {
   ENTER(__FUNCTION__);
   assert(id);
   assert(length);
-  int ret = -1;
+  int ret = PULSE_ERROR;
   *length = 0;
   pa_operation *o = NULL;
 
@@ -382,7 +393,7 @@ _pulse_free(AudioID *id, size_t* length) {
 
   SHOW("_pulse_free: %s\n", "pa_stream_writable_size");
 
-  ret = 0;
+  ret = PULSE_OK;
 
   /* If this function is called twice with no _pulse_write() call in
    * between this means we should trigger the playback */
@@ -462,24 +473,24 @@ _pulse_write(AudioID *id, void* ptr, int length)
 {
   ENTER(__FUNCTION__);
 
-  int ret = -1;
+  int ret = PULSE_ERROR;
   assert(id);
 
   SHOW("_pulse_write > length=%d\n", length);
 
-  CHECK_CONNECTED(id, -1);
+  CHECK_CONNECTED(id, PULSE_ERROR);
 
   pa_threaded_mainloop_lock(id->pulse_mainloop);
   CHECK_DEAD_GOTO(id, fail, 1);
 
   if (pa_stream_write(id->pulse_stream, ptr, length, NULL, PA_SEEK_RELATIVE, (pa_seek_mode_t)0) < 0) {
     ERR("pa_stream_write() failed: %s", pa_strerror(pa_context_errno(id->pulse_context)));
-    ret = -1;
+    ret = PULSE_ERROR;
     goto fail;
   }
     
   id->pulse_do_trigger = 0;
-  ret = 0;
+  ret = PULSE_OK;
 
  fail:    
   pa_threaded_mainloop_unlock(id->pulse_mainloop);
@@ -490,15 +501,15 @@ static int
 _drain(AudioID *id) 
 {
   pa_operation *o = NULL;
-  int ret = -1;
+  int ret = PULSE_ERROR;
   assert(id);
 
   ENTER(__FUNCTION__);
 
-  CHECK_CONNECTED(id, -1);
+  CHECK_CONNECTED(id, PULSE_ERROR);
 
   pa_threaded_mainloop_lock(id->pulse_mainloop);
-  CHECK_DEAD_GOTO(id, fail, 1); //TBD 0 instead?
+  CHECK_DEAD_GOTO(id, fail, 1); /* TBD 0 instead? */
 
   SHOW("Draining...\n","");
   DISPLAY_ID(id, "pa_threaded_mainloop_lock");
@@ -522,7 +533,7 @@ _drain(AudioID *id)
     ERR("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(id->pulse_context)));
   } 
   else {
-    ret = 0;
+    ret = PULSE_OK;
   }
 
  fail:
@@ -601,7 +612,7 @@ _pulse_get_sample(const AudioTrack* track, pa_sample_spec* ss)
     ss->format = PA_SAMPLE_U8; 
   }else{
     ERR("Unsupported sound data format, track.bits = %d\n", track->bits);
-    return -1;
+    return PULSE_ERROR;
   }
 
   ss->rate = track->sample_rate;
@@ -614,19 +625,22 @@ _pulse_get_sample(const AudioTrack* track, pa_sample_spec* ss)
 
   if (!pa_sample_spec_valid(ss)) {      
     ERR("Sample spec not valid!\n","");
-    return -1;
+    return PULSE_ERROR;
   }
 
   SHOW("Sample spec valid\n","");
-  return 0;
+  return PULSE_OK;
 }
 
+
+/* return PULSE_OK, PULSE_ERROR, PULSE_NO_CONNECTION */
 static int 
 _pulse_open(AudioID *id, pa_sample_spec* ss) 
 {
   ENTER(__FUNCTION__);
   
   pa_operation *o = NULL;
+  int ret = PULSE_ERROR;
 
   assert(id && ss);
 
@@ -661,6 +675,7 @@ _pulse_open(AudioID *id, pa_sample_spec* ss)
   SHOW_TIME("pa_context_connect (call)");
   if (pa_context_connect(id->pulse_context, id->pulse_server, (pa_context_flags_t)0, NULL) < 0) {
     ERR("Failed to connect to server: %s", pa_strerror(pa_context_errno(id->pulse_context)));
+    ret = PULSE_NO_CONNECTION;
     goto unlock_and_fail;
   }
 
@@ -676,6 +691,9 @@ _pulse_open(AudioID *id, pa_sample_spec* ss)
 
   if (pa_context_get_state(id->pulse_context) != PA_CONTEXT_READY) {
     ERR("Failed to connect to server: %s", pa_strerror(pa_context_errno(id->pulse_context)));
+    ret = PULSE_NO_CONNECTION;
+    if (id->pulse_mainloop)
+      pa_threaded_mainloop_stop(id->pulse_mainloop);
     goto unlock_and_fail;
   }
 
@@ -694,7 +712,7 @@ _pulse_open(AudioID *id, pa_sample_spec* ss)
   a_attr.tlength = id->pulse_target_length;
   a_attr.prebuf = id->pulse_pre_buffering;
   a_attr.minreq = id->pulse_min_request;
-  a_attr.fragsize = 882;
+  a_attr.fragsize = 0;
 
   SHOW("attr: maxlength=%i, tlength=%i, prebuf=%i, minreq=%i, fragsize=%i\n",
        a_attr.maxlength,
@@ -767,7 +785,7 @@ _pulse_open(AudioID *id, pa_sample_spec* ss)
 
   SHOW("_pulse_open (ret true)\n","");
 
-  return 0;
+  return PULSE_OK;
 
  unlock_and_fail:
 
@@ -778,11 +796,33 @@ _pulse_open(AudioID *id, pa_sample_spec* ss)
     
  fail:
 
-  _pulse_close(id);
+  if (ret == PULSE_NO_CONNECTION) {
+    if (id->pulse_context) {
+      SHOW_TIME("pa_context_disconnect (call)");
+      pa_context_disconnect(id->pulse_context);
+      pa_context_unref(id->pulse_context);
+      id->pulse_context = NULL;
+    }
+  
+    if (id->pulse_mainloop) {
+      SHOW_TIME("pa_threaded_mainloop_free (call)");
+      pa_threaded_mainloop_free(id->pulse_mainloop);
+      id->pulse_mainloop = NULL;
+    }
+  
+    if (id->pulse_server)
+      {
+	free(id->pulse_server);
+	id->pulse_server = NULL;
+      }
+  } 
+  else {
+    _pulse_close(id);
+  }
 
-  SHOW("_pulse_open (ret false)\n","");
+  SHOW("_pulse_open (ret %d)\n", ret);
     
-  return -1;
+  return ret;
 }
 
 static int 
@@ -791,6 +831,7 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
   ENTER(__FUNCTION__);
   pa_sample_spec ss;
   pa_operation *o = NULL;
+  int ret = PULSE_ERROR;
 
   assert(id);
 
@@ -807,7 +848,7 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
     ss.format = PA_SAMPLE_U8; 
   }else{
     ERR("Unsupported sound data format, track.bits = %d\n", track.bits);
-    return -1;
+    return PULSE_ERROR;
   }
 
   ss.rate = track.sample_rate;
@@ -820,7 +861,7 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
 
   if (!pa_sample_spec_valid(&ss)) {      
     ERR("Sample spec not valid!\n","");
-    return -1;
+    return PULSE_ERROR;
   }
 
   SHOW("Sample spec valid\n","");
@@ -851,6 +892,7 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
   SHOW_TIME("pa_context_connect (call)");
   if (pa_context_connect(id->pulse_context, id->pulse_server, (pa_context_flags_t)0, NULL) < 0) {
     ERR("Failed to connect to server: %s", pa_strerror(pa_context_errno(id->pulse_context)));
+    ret = PULSE_NO_CONNECTION;
     goto unlock_and_fail;
   }
 
@@ -866,6 +908,9 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
 
   if (pa_context_get_state(id->pulse_context) != PA_CONTEXT_READY) {
     ERR("Failed to connect to server: %s", pa_strerror(pa_context_errno(id->pulse_context)));
+    ret = PULSE_NO_CONNECTION;
+    if (id->pulse_mainloop)
+      pa_threaded_mainloop_stop(id->pulse_mainloop);
     goto unlock_and_fail;
   }
 
@@ -884,7 +929,7 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
   a_attr.tlength = id->pulse_target_length;
   a_attr.prebuf = id->pulse_pre_buffering;
   a_attr.minreq = id->pulse_min_request;
-  a_attr.fragsize = 882;
+  a_attr.fragsize = 0;
 
   SHOW("attr: maxlength=%i, tlength=%i, prebuf=%i, minreq=%i, fragsize=%i\n",
        a_attr.maxlength,
@@ -957,7 +1002,7 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
 
   SHOW("_pulse_set_sample (ret true)\n","");
 
-  return 0;
+  return PULSE_OK;
 
  unlock_and_fail:
 
@@ -967,15 +1012,174 @@ _pulse_set_sample(AudioID *id, AudioTrack track)
   pa_threaded_mainloop_unlock(id->pulse_mainloop);
     
  fail:
-
-  _pulse_close(id);
+  if (ret == PULSE_NO_CONNECTION) {
+    if (id->pulse_context) {
+      SHOW_TIME("pa_context_disconnect (call)");
+      pa_context_disconnect(id->pulse_context);
+      pa_context_unref(id->pulse_context);
+      id->pulse_context = NULL;
+    }
+  
+    if (id->pulse_mainloop) {
+      SHOW_TIME("pa_threaded_mainloop_free (call)");
+      pa_threaded_mainloop_free(id->pulse_mainloop);
+      id->pulse_mainloop = NULL;
+    }
+  
+    if (id->pulse_server)
+      {
+	free(id->pulse_server);
+	id->pulse_server = NULL;
+      }
+  } 
+  else {
+    _pulse_close(id);
+  }
 
   SHOW("_pulse_set_sample (ret false)\n","");
     
-  return -1;
+  return ret;
 }
 
 
+/* _pulse_timeout_thread
+   Close the PulseAudio connection after a while of inactivaty (PULSE_INFINITE_TIMEOUT_IN_SEC).
+
+   The timeout is started by pulse_play as soon as the stream has been played:
+   pulse_play indicates the current time in time_start and increments the semaphore.
+
+   If pulse_play is called again, it simply resets the time_start value.
+   
+   The semaphore incremented by pulse_play wakes up _pulse_timeout_thread which then 
+   stores the time_start value and waits for the next step (timeout occurence or semaphore).
+
+   If timeout occurs, the time_start value is compared to the previous one. 
+   If equal, _pulse_close is called. Otherwise, the timeout is ignored since pulse_play has 
+   been called again.
+*/
+static void* 
+_pulse_timeout_thread(void* the_id)
+{
+  ENTER(__FUNCTION__);
+
+  struct timespec ts;
+  struct timeval tv;	
+  int time_ref;	
+  AudioID * id = (AudioID *)the_id;
+  int a_start_is_required=0;
+  t_pulse_timeout* a_timeout = &(id->pulse_timeout);
+
+  assert (gettimeofday(&tv, NULL) != -1);
+  ts.tv_sec = tv.tv_sec;
+  ts.tv_nsec = tv.tv_usec*1000;
+
+  a_timeout->my_close_is_imminent = 0;
+
+  int i=0;
+  while(1) {
+    int err = PULSE_OK;
+    
+    SHOW_TIME("sem_wait/sem_timedwait");
+
+    if (a_timeout->my_close_is_imminent) {
+      while ((err = sem_timedwait(&(a_timeout->my_sem), &ts)) == -1 
+	     && errno == EINTR) {
+	continue;
+      }
+      SHOW_TIME("sem_timedwait (ret)");
+    } 
+    else {
+      SHOW("%s > wait for a request\n", __FUNCTION__);    
+      while ((err = sem_wait(&(a_timeout->my_sem))) == -1 
+	     && errno == EINTR) {
+	continue;
+      }
+      SHOW_TIME("sem_wait (ret)");
+    }
+
+    time_ref = a_timeout->my_time_start;
+
+    assert (gettimeofday(&tv, NULL) != -1);
+
+    if (err == PULSE_OK) { /* got semaphore */
+      if (time_ref) {
+	SHOW("%s > compute timeout (my_time_start=%d)\n", __FUNCTION__, time_ref);
+	int a_usec = tv.tv_usec + PULSE_TIMEOUT_IN_USEC;
+
+	tv.tv_sec += a_usec / 1000000;
+	tv.tv_usec = a_usec % 1000000;
+
+	ts.tv_sec = tv.tv_sec;	  
+	ts.tv_nsec = tv.tv_usec*1000;
+
+	a_timeout->my_close_is_imminent = 1;
+	a_timeout->my_time_start_old = time_ref;
+      }
+      else {
+	SHOW("%s > time_ref=0!\n", __FUNCTION__);
+	a_timeout->my_close_is_imminent = 0;
+      }
+    }
+    else if (a_timeout->my_close_is_imminent) { /* timeout */
+      if (a_timeout->my_time_start_old == time_ref) {	
+	SHOW("%s > timeout!\n", __FUNCTION__);
+	int a_status;
+	pthread_mutex_t* a_mutex = &id->pulse_mutex;
+
+	if ((a_status=pthread_mutex_lock(a_mutex))) {
+	  ERR("Error: pulse_mutex lock=%d (%s)\n", a_status, __FUNCTION__);
+	} 
+	else {
+	  _pulse_close(id);	
+	  pthread_mutex_unlock(a_mutex);
+	}
+      } else {
+	SHOW("%s > my_time_start has changed (old=%d, current=%d)\n", __FUNCTION__, a_timeout->my_time_start_old, time_ref);
+      }
+      a_timeout->my_close_is_imminent = 0;
+    }
+  }
+
+  SHOW_TIME("_pulse_timeout (ret)");
+  
+  return NULL;
+}
+
+/* start timeout */
+static void 
+_pulse_timeout_start(AudioID *id)
+{
+  ENTER(__FUNCTION__);
+
+  assert(id);
+  t_pulse_timeout* a_timeout = &(id->pulse_timeout);
+
+  struct timeval tv;	
+
+  assert (gettimeofday(&tv, NULL) != -1);
+  
+  /* convert the time in ms.
+     the limit (currently 5s) must be higher than PULSE_TIMEOUT_IN_USEC! */
+  a_timeout->my_time_start = tv.tv_usec/1000 + 1000*(tv.tv_sec%4);
+  SHOW("my_time_start=%d\n", a_timeout->my_time_start);
+
+  assert( sem_post(&(a_timeout->my_sem)) != -1);
+}
+
+/* stop timeout */
+static void 
+_pulse_timeout_stop(AudioID *id)
+{
+  ENTER(__FUNCTION__);
+
+  assert(id);
+  t_pulse_timeout* a_timeout = &(id->pulse_timeout);
+  a_timeout->my_time_start = 0;
+  SHOW("my_time_start=%d\n", a_timeout->my_time_start);
+}
+
+
+
 /* external services */
 
 
@@ -999,16 +1203,29 @@ pulse_open(AudioID *id, void **pars)
 
   if (id == NULL){
     ERR("Can't open PulseAudio sound output, invalid AudioID structure.\n","");
-    return -1;
+    return PULSE_ERROR;
   }
 
   if (pars[0] == NULL){
     ERR("Can't open PulseAudio sound output, missing parameters in argument.\n","");
-    return -1;
+    return PULSE_ERROR;
   }
     
   pthread_mutex_init( &id->pulse_mutex, (const pthread_mutexattr_t *)NULL);
 
+  assert(-1 != sem_init(&(id->pulse_timeout.my_sem), 0, 0));
+  pthread_attr_t a_attrib;    
+  if (pthread_attr_init (& a_attrib)
+      || pthread_attr_setdetachstate(&a_attrib, PTHREAD_CREATE_JOINABLE)
+      || pthread_create( &(id->pulse_timeout.my_thread), 
+			 & a_attrib, 
+			 _pulse_timeout_thread, 
+			 (void*)id))
+    {
+      assert(0);
+    }
+  pthread_attr_destroy(&a_attrib);
+
   id->pulse_context = NULL;
   id->pulse_stream = NULL;
   id->pulse_mainloop = NULL;
@@ -1029,14 +1246,16 @@ pulse_open(AudioID *id, void **pars)
     id->pulse_server = NULL;
   }
   
-  id->pulse_max_length = (int)pars[1];
-  id->pulse_target_length = (int)pars[2];
-  id->pulse_pre_buffering = (int)pars[3];
-  id->pulse_min_request = (int)pars[4];
+  /* The following cast is verified correct despite
+   gcc warnings */
+  id->pulse_max_length = (int) pars[1];
+  id->pulse_target_length = (int) pars[2];
+  id->pulse_pre_buffering = (int) pars[3];
+  id->pulse_min_request = (int) pars[4];
 
   SHOW("PulseAudio sound output opened\n","");
 
-  return 0;
+  return PULSE_OK;
 }
 
 int
@@ -1046,36 +1265,38 @@ pulse_play(AudioID *id, AudioTrack track)
 
   int bytes_per_sample;
   int num_bytes;
-  int ret = 0;
-  unsigned int a_total_free_mem;
+  int ret = PULSE_OK;
+  size_t a_total_free_mem;
   pa_sample_spec ss;
 
   if (id == NULL){
     ERR("Invalid device passed to %s()\n",__FUNCTION__);
-    return -1;
+    return PULSE_ERROR;
   }
 
+  _pulse_timeout_stop(id);
+
   /* Is it not an empty track? */
   /* Passing an empty track is not an error */
   if (track.samples == NULL) {
     ERR("Empty track!\n","");
-    return 0;
+    return PULSE_OK;
   }
 
-  if (_pulse_get_sample(&track, &ss) == -1) {
+  if (_pulse_get_sample(&track, &ss) == PULSE_ERROR) {
     ERR("Erroneous track!\n","");
-    return -1;
+    return PULSE_ERROR;
   }
 
   int a_status = pthread_mutex_lock(&id->pulse_mutex);
   if (a_status) {
     ERR("Error: pulse_mutex lock=%d (%s)\n", a_status, __FUNCTION__);
-    return -1;
+    return PULSE_ERROR;
     }
 
   if (id->pulse_mainloop) {
     if (id->pulse_stop_required) {
-      ret = 0;
+      ret = PULSE_OK;
       goto terminate;
     }
 
@@ -1089,11 +1310,11 @@ pulse_play(AudioID *id, AudioTrack track)
 
   if (!id->pulse_mainloop) {
     if (id->pulse_stop_required) {
-      ret = 0;
+      ret = PULSE_OK;
       goto terminate;
     }
     ret = _pulse_open(id, &ss);
-    if (ret != 0) {
+    if (ret != PULSE_OK) {
       goto terminate;
     }
   }
@@ -1105,7 +1326,7 @@ pulse_play(AudioID *id, AudioTrack track)
     bytes_per_sample = 1;    
   }else{
     ERR("Unsupported sound data format, track.bits = %d\n", track.bits);
-    return -1;
+    return PULSE_ERROR;
   }
 
   /* Loop until all samples are played on the device. */
@@ -1143,8 +1364,14 @@ pulse_play(AudioID *id, AudioTrack track)
     usleep(10000);
   }
 
+  SHOW_TIME("pulse_play > _drain");
+  _drain(id);
+
+  _pulse_timeout_start(id);
+
  terminate:
   pthread_mutex_unlock(&id->pulse_mutex);
+  SHOW("pulse_play: ret=%d", ret);
   SHOW_TIME("pulse_play (ret)");
   return ret;
 }
@@ -1156,7 +1383,7 @@ pulse_stop(AudioID *id)
 
   if (id == NULL){
     ERR("Invalid device passed to %s\n",__FUNCTION__);
-    return -1;
+    return PULSE_ERROR;
   }
 
   id->pulse_stop_required = 1;
@@ -1164,7 +1391,7 @@ pulse_stop(AudioID *id)
   if (a_status) {
     id->pulse_stop_required = 0;
     ERR("Error: pulse_mutex lock=%d (%s)\n", a_status, __FUNCTION__);
-    return -1;
+    return PULSE_ERROR;
     }
 
   _drain(id);
@@ -1173,7 +1400,7 @@ pulse_stop(AudioID *id)
   a_status = pthread_mutex_unlock(&id->pulse_mutex);
   SHOW_TIME("pulse_stop (ret)");
 
-  return 0;
+  return PULSE_OK;
 }
 
 int
@@ -1186,22 +1413,29 @@ pulse_close(AudioID *id)
 
   if (id == NULL){
     ERR("Invalid device passed to %s\n",__FUNCTION__);
-    return -1;
+    return PULSE_ERROR;
   }
 
   a_mutex = &id->pulse_mutex;
   a_status = pthread_mutex_lock(a_mutex);
+
   if (a_status) {
     ERR("Error: pulse_mutex lock=%d (%s)\n", a_status, __FUNCTION__);
-    return -1;
+    return PULSE_ERROR;
     }
 
   _pulse_close(id);
 
+  SHOW_TIME("timeout thread cancel");
+  pthread_cancel(id->pulse_timeout.my_thread);
+  pthread_join(id->pulse_timeout.my_thread, NULL);
+  sem_destroy(&(id->pulse_timeout.my_sem));
+
+  SHOW_TIME("unlock mutex");
   a_status = pthread_mutex_unlock(a_mutex);
   pthread_mutex_destroy(a_mutex);
 
-  return 0;
+  return PULSE_OK;
 }
 
 int
@@ -1213,7 +1447,7 @@ pulse_set_volume(AudioID*id, int volume)
 
   if ((volume > 100) || (volume < -100)){
     ERR("Requested volume out of range (%d)", volume);
-    return -1;
+    return PULSE_ERROR;
   }
 
   if (id->pulse_connected) {
@@ -1241,7 +1475,7 @@ pulse_set_volume(AudioID*id, int volume)
   if (id->pulse_connected)
     pa_threaded_mainloop_unlock(id->pulse_mainloop);
     
-  return 0;
+  return PULSE_OK;
 }
 
 /* Provide the PulseAudio backend */
