@@ -22,7 +22,7 @@
  * @author Lukas Loehrer
  * Based on ibmtts.c.
  *
- * $Id: espeak.c,v 1.6 2007-11-12 14:46:43 hanke Exp $
+ * $Id: espeak.c,v 1.8 2008-02-08 10:01:09 hanke Exp $
  */
 
 /* < Includes*/
@@ -410,7 +410,7 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
 
 	if (0 != module_write_data_ok(data)) return FATAL_ERROR;
 
-	DBG("Espeak: Requested data: |%s| %d %d", data, msgtype, bytes);
+	DBG("Espeak: Requested data: |%s| %d %ld", data, msgtype, bytes);
 
 	espeak_state_reset();
 	espeak_state = BEFORE_SYNTH;
@@ -673,7 +673,7 @@ espeak_set_rate(signed int rate)
 {
 	assert(rate >= -100 && rate <= +100);
 	int speed;
-	int normal_rate = 170, max_rate = 370, min_rate = 80;
+	int normal_rate = 170, max_rate = 390, min_rate = 80;
     
 	if (rate < 0)
 		speed = normal_rate + (normal_rate - min_rate) * rate / 100;
