@@ -34,12 +34,10 @@ class _SSIPClientTest(unittest.TestCase):
 
 class AutomaticTest(_SSIPClientTest):
     """A set of tests which may be evaluated automatically.
-
     Please put all tests which require a user to listen to their output to the
     VoiceTest below.
 
     """
-    
     def test_callbacks(self):
         # TODO: This needs to be fixed. There is no guarantee that
         # the message will start in one second nor is there any
@@ -49,6 +47,7 @@ class AutomaticTest(_SSIPClientTest):
         # received on time. Also the combination cancel/end does not have
         # to work as expected and SD and the interface can still be ok.
         # -- Hynek Hanke
+        self._client.set_output_module('flite')
         called = {CallbackType.BEGIN: [],
                   CallbackType.CANCEL: [],
                   CallbackType.END: []}
@@ -59,13 +58,16 @@ class AutomaticTest(_SSIPClientTest):
                            callback=lambda type: called[type].append('msg2'))
         time.sleep(1)
         self._client.cancel()
+        self._client.speak("Hi.",
+                           callback=lambda type: called[type].append('msg3'))
         # Wait for pending events...
-        time.sleep(1)
+        time.sleep(3)
         started, canceled, ended = [called[t] for t in (CallbackType.BEGIN,
-                                                      CallbackType.CANCEL,
-                                                      CallbackType.END)]
-        assert started == ['msg1'] and ended == [] and \
-               'msg1' in canceled and 'msg2' in canceled, \
+                                                        CallbackType.CANCEL,
+                                                        CallbackType.END)]
+        assert started == ['msg1', 'msg3'] and ended == ['msg3'] and \
+               'msg1' in canceled and 'msg2' in canceled and \
+               'msg3' not in canceled, \
                (called,
                 "This failure only indicates a possible error.  The test "
                 "depends on proper timing and results may warry depending "
@@ -116,7 +118,18 @@ class VoiceTest(_SSIPClientTest):
         c.char("a")
         c.key("shift_b")
         c.sound_icon("empty")
-
+        
+    def test_lists(self):
+         c = self._client
+         for module in  c.list_output_modules():
+             c.set_output_module(module)
+             print "**", module
+             c.speak(module +"using default voice")
+             for name, lang, dialect in c.list_synthesis_voices():
+                 print " -", module, name, lang, dialect
+                 c.set_synthesis_voice(name)
+                 c.speak(module +" using voice "+ name)
+        
 
 if __name__ == '__main__':
     unittest.main()
