@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: options.h,v 1.5 2006/07/11 16:12:27 hanke Exp $
+ * $Id: options.h,v 1.6 2007-11-21 14:19:13 hanke Exp $
  */
 
 #include <getopt.h>
@@ -29,12 +29,13 @@ static struct option spd_long_options[] = {
     {"log-level", 1, 0, 'l'},
     {"port", 1, 0, 'p'},
     {"pid-file", 1, 0, 'P'},
+    {"config-file", 1, 0, 'C'},
     {"version", 0, 0, 'v'},
     {"help", 0, 0, 'h'},
     {0, 0, 0, 0}
 };
 
-static char* spd_short_options = "dsl:p:P:vh";
+static char* spd_short_options = "dsl:p:P:C:vh";
 
 void options_print_help(char *argv[]);
 void options_print_version(void);
