@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: parse.c,v 1.68 2007-06-21 20:29:52 hanke Exp $
+ * $Id: parse.c,v 1.71 2008-02-11 14:01:42 hanke Exp $
  */
 
 #include <ctype.h>
@@ -76,8 +76,8 @@ parse(const char *buf, const int bytes, const int fd)
     if ((buf == NULL) || (bytes == 0)){
         if(SPEECHD_DEBUG) FATAL("invalid buffer for parse()\n");
         return strdup(ERR_INTERNAL);
-    }	
-   	
+    }
+ 
     /* First the condition that we are not in data mode and we
      * are awaiting commands */
     if (SpeechdSocket[fd].awaiting_data == 0){
@@ -138,11 +138,13 @@ parse(const char *buf, const int bytes, const int fd)
          enddata:
         /* In the end of the data flow we got a "\r\n.\r\n" command. */
         MSG(5,"Buffer: |%s| bytes:", buf, bytes);
-        if(((bytes >= 5)&&((!strncmp(buf, "\r\n.\r\n", bytes))))||(end_data == 1)
-           ||((bytes == 3)&&(!strncmp(buf, ".\r\n", bytes)))){
+
+        if(((bytes >= 5) && ((!strncmp(buf, "\r\n.\r\n", bytes))))||(end_data == 1)
+           ||((bytes == 3) && (!strncmp(buf, ".\r\n", bytes)))){
 
             MSG(5,"Finishing data");
             end_data = 0;
+
             /* Set the flag to command mode */
             MSG(5, "Switching back to command mode...");
             SpeechdSocket[fd].awaiting_data = 0;
@@ -152,7 +154,17 @@ parse(const char *buf, const int bytes, const int fd)
 	    /* TODO: Remove? */
             if ((bytes == 3) && (SpeechdSocket[fd].o_bytes > 2)) SpeechdSocket[fd].o_bytes -= 2;
 
+	    /* Check if message contains any data */
             if (SpeechdSocket[fd].o_bytes == 0) return strdup(OK_MSG_CANCELED);
+
+	    /* Check buffer for proper UTF-8 encoding */
+	    if (!g_utf8_validate(SpeechdSocket[fd].o_buf->str, SpeechdSocket[fd].o_bytes, NULL))
+	      {
+		MSG(3, "ERROR: Invalid character encoding on input (failed UTF-8 validation)");
+		MSG(3, "Rejecting this message.");
+		return strdup(ERR_INVALID_ENCODING);
+	      }
+
             new = (TSpeechDMessage*) spd_malloc(sizeof(TSpeechDMessage));
             new->bytes = SpeechdSocket[fd].o_bytes;
 	    new->buf = (char*) spd_malloc(new->bytes + 1);
@@ -187,7 +199,7 @@ parse(const char *buf, const int bytes, const int fd)
         {
             int real_bytes;
             if(bytes>=5){
-                if(pos = strstr(buf,"\r\n.\r\n")){	
+	      if ( (pos = strstr(buf,"\r\n.\r\n")) ){	
                     real_bytes=pos-buf;
                     end_data=1;		
                     MSG(5,"Command in data caught");
@@ -374,7 +386,7 @@ parse_set(const char *buf, const int bytes, const int fd)
 {
     int who;                    /* 0 - self, 1 - uid specified, 2 - all */
     int uid;                    /* uid of the client (only if who == 1) */
-    int ret;  
+    int ret = -1; // =-1 has no effect but avoids gcc warning  
     char *set_sub;
     char *who_s;
 
@@ -749,6 +761,16 @@ parse_general_event(const char *buf, const int bytes, const int fd, EMessageType
 	return strdup(ERR_MISSING_PARAMETER);
     }
 
+    /* Check for proper UTF-8 */
+    /* Check buffer for proper UTF-8 encoding */
+    if (!g_utf8_validate(buf, bytes, NULL))
+      {
+	MSG(3, "ERROR: Invalid character encoding on event input (failed UTF-8 validation)");
+	MSG(3, "Rejecting this event (char/key/sound_icon).");
+	return strdup(ERR_INVALID_ENCODING);
+      }
+
+
     msg = (TSpeechDMessage*) spd_malloc(sizeof(TSpeechDMessage));
     msg->bytes = strlen(param);
     msg->buf = strdup(param);
@@ -938,7 +960,7 @@ deescape_dot(char *otext)
 
     MSG2(6, "escaping", "Altering text (I): |%s|", ntext->str);
 
-    while (seq = strstr(otext, "\r\n..\r\n")){
+    while ( (seq = strstr(otext, "\r\n..\r\n")) ){
         *seq = 0;
         g_string_append(ntext, otext);
         g_string_append(ntext, "\r\n.\r\n");
@@ -995,8 +1017,9 @@ get_param(const char *buf, const int n, const int bytes, const int lower_case)
 {
     char* param;
     char* par;
-    int i, y, z;
+    int i, y, z = 0;
 
+    assert (bytes != 0);
     param = (char*) spd_malloc(bytes);
     assert(param != NULL);
 	
