@@ -19,7 +19,7 @@
   * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   * Boston, MA 02110-1301, USA.
   *
-  * $Id: server.c,v 1.81 2007/02/27 17:33:23 hanke Exp $
+  * $Id: server.c,v 1.84 2008-02-08 10:01:09 hanke Exp $
   */
 
 #include "speechd.h"
@@ -70,7 +70,6 @@ queue_message(TSpeechDMessage *new, int fd, int history_flag,
     int id;
     GList *element;
 
-
     /* Check function parameters */
     if (new == NULL) return -1;
     if (new->buf == NULL) return -1;
@@ -98,6 +97,7 @@ queue_message(TSpeechDMessage *new, int fd, int history_flag,
 	COPY_SET_STR(client_name);
 	COPY_SET_STR(output_module);
 	COPY_SET_STR(language);
+	COPY_SET_STR(synthesis_voice);
 
 	/* And we set the global id (note that this is really global, not
 	 * depending on the particular client, but unique) */
@@ -145,6 +145,7 @@ queue_message(TSpeechDMessage *new, int fd, int history_flag,
 
     pthread_mutex_lock(&element_free_mutex);
     /* Put the element new to queue according to it's priority. */
+    check_locked(&element_free_mutex);    
     switch(settings->priority){
     case 1: MessageQueue->p1 = g_list_append(MessageQueue->p1, new); 
         break;
