@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: set.c,v 1.41 2006/07/11 16:12:27 hanke Exp $
+ * $Id: set.c,v 1.43 2007-11-05 09:17:56 hanke Exp $
  */
 
 #include <fnmatch.h>
@@ -153,6 +153,10 @@ set_voice_uid(int uid, char *voice)
     else if (!strcmp(voice, "child_female")) settings->voice = CHILD_FEMALE;
     else return 1;
 
+    if (settings->synthesis_voice != NULL){
+      free(settings->synthesis_voice);
+      settings->synthesis_voice = NULL;
+    }
     return 0;
 }
 
@@ -226,6 +230,24 @@ set_language_uid(int uid, char *language)
     return 0;
 }
 
+SET_SELF_ALL(char*, synthesis_voice);
+
+int
+set_synthesis_voice_uid(int uid, char *synthesis_voice)
+{
+    TFDSetElement *settings;
+
+    settings = get_client_settings_by_uid(uid);
+    if (settings == NULL) return 1;
+	
+    SET_PARAM_STR(synthesis_voice);        
+
+    /* Delete ordinary voice settings so that we don't mix */
+    settings->voice = NO_VOICE;
+
+    return 0;
+}
+
 #define CHECK_SET_PAR(name, ival) \
    if (cl_set->val.name != ival){ set->name = cl_set->val.name; \
             MSG(4,"parameter " #name " set to %d", cl_set->val.name); }
@@ -308,6 +330,12 @@ set_output_module_uid(int uid, char* output_module)
 
     SET_PARAM_STR(output_module);
 
+    /* Delete synth_voice since it is module specific */
+    if (settings->synthesis_voice != NULL){
+      free(settings->synthesis_voice);
+      settings->synthesis_voice = NULL;
+    }
+
     return 0;
 }
 
@@ -405,6 +433,7 @@ default_fd_set(void)
 	new->output_module = spd_strdup(GlobalFDSet.output_module);
 	new->client_name = spd_strdup(GlobalFDSet.client_name); 
 	new->voice = GlobalFDSet.voice;
+	new->synthesis_voice = NULL;
 	new->spelling_mode = GlobalFDSet.spelling_mode;         
 	new->cap_let_recogn = GlobalFDSet.cap_let_recogn;      
 
